#version 330

//---------------------------------------------------------------------------
uniform sampler2D gSampler;
uniform vec2 gSampleSize;
uniform float gFXAASpanMax = 8.0;
uniform float gFXAAReduceMul = 1.0/8.0;
//---------------------------------------------------------------------------
in vec4 posPos;
in vec2 TexCoords;
//---------------------------------------------------------------------------
out vec4 outColor;
//---------------------------------------------------------------------------
#define FxaaInt2 ivec2
#define FxaaFloat2 vec2
#define FxaaTexLod0(t, p) textureLod(t, p, 0.0)
#define FxaaTexOff(t, p, o, r) textureLodOffset(t, p, 0.0, o)
//---------------------------------------------------------------------------
// posPos: Output of FxaaVertexShader interpolated across screen.
// tex: Input texture.
// rcpFrame: Constant {1.0/frameWidth, 1.0/frameHeight}.
vec3 FxaaPixelShader( vec4 posPos, sampler2D tex, vec2 rcpFrame) 
{   
	/*---------------------------------------------------------*/
	#define FXAA_REDUCE_MIN   (1.0/128.0)
	//#define gFXAAReduceMul   (1.0/8.0)
	//#define gFXAASpanMax     8.0
	/*---------------------------------------------------------*/
	vec3 rgbNW = FxaaTexLod0(tex, posPos.zw).xyz;
	vec3 rgbNE = FxaaTexOff(tex, posPos.zw, FxaaInt2(1,0), rcpFrame.xy).xyz;
	vec3 rgbSW = FxaaTexOff(tex, posPos.zw, FxaaInt2(0,1), rcpFrame.xy).xyz;
	vec3 rgbSE = FxaaTexOff(tex, posPos.zw, FxaaInt2(1,1), rcpFrame.xy).xyz;
	vec3 rgbM  = FxaaTexLod0(tex, posPos.xy).xyz;
	/*---------------------------------------------------------*/
	vec3 luma = vec3(0.299, 0.587, 0.114);
	float lumaNW = dot(rgbNW, luma);
	float lumaNE = dot(rgbNE, luma);
	float lumaSW = dot(rgbSW, luma);
	float lumaSE = dot(rgbSE, luma);
	float lumaM  = dot(rgbM,  luma);
	/*---------------------------------------------------------*/
	float lumaMin = min(lumaM, min(min(lumaNW, lumaNE), min(lumaSW, lumaSE)));
	float lumaMax = max(lumaM, max(max(lumaNW, lumaNE), max(lumaSW, lumaSE)));
	/*---------------------------------------------------------*/
	vec2 dir; 
	dir.x = -((lumaNW + lumaNE) - (lumaSW + lumaSE));
	dir.y =  ((lumaNW + lumaSW) - (lumaNE + lumaSE));
	/*---------------------------------------------------------*/
	float dirReduce = max( (lumaNW + lumaNE + lumaSW + lumaSE) * (0.25 * gFXAAReduceMul), FXAA_REDUCE_MIN);
	float rcpDirMin = 1.0 / (min(abs(dir.x), abs(dir.y)) + dirReduce);
	dir = min(FxaaFloat2( gFXAASpanMax,  gFXAASpanMax), 
			max(FxaaFloat2(-gFXAASpanMax, -gFXAASpanMax), 
			dir * rcpDirMin)) * rcpFrame.xy;
	/*--------------------------------------------------------*/
	vec3 rgbA = (1.0/2.0) * ( FxaaTexLod0(tex, posPos.xy + dir * (1.0/3.0 - 0.5)).xyz +
								FxaaTexLod0(tex, posPos.xy + dir * (2.0/3.0 - 0.5)).xyz);
	vec3 rgbB = rgbA * (1.0/2.0) + (1.0/4.0) * ( FxaaTexLod0(tex, posPos.xy + dir * (0.0/3.0 - 0.5)).xyz +
													FxaaTexLod0(tex, posPos.xy + dir * (3.0/3.0 - 0.5)).xyz);
	float lumaB = dot(rgbB, luma);
	if((lumaB < lumaMin) || (lumaB > lumaMax)) 
	{
		return rgbA;
	}

	return rgbB; 
}
//---------------------------------------------------------------------------
vec4 PostFX(sampler2D tex, vec2 uv, float time)
{
	vec4 c = vec4(0.0);
	vec2 rcpFrame = vec2(1.0/gSampleSize.x, 1.0/gSampleSize.y);
	c.rgb = FxaaPixelShader(posPos, tex, rcpFrame);
	//c.rgb = 1.0 - texture2D(tex, posPos.xy).rgb;
	c.a = 1.0;
	return c;
}
//---------------------------------------------------------------------------
void main() 
{ 
	outColor = PostFX(gSampler, TexCoords, 0.0);
}
