#version 330

//---------------------------------------------------------------------------
uniform sampler2D gSampler;
uniform sampler2D gLensColours;
uniform int gGhosts;
uniform float gGhostDispersal;
//---------------------------------------------------------------------------
noperspective in vec2 TexCoords;
//---------------------------------------------------------------------------
out vec4 outColor;
//---------------------------------------------------------------------------
vec4 TextureDistorted( in sampler2D tex, in vec2 texcoord, in vec2 direction, in vec3 distortion ) 
{
	return vec4( texture(tex, texcoord + direction * distortion.r).r,
				 texture(tex, texcoord + direction * distortion.g).g,
				 texture(tex, texcoord + direction * distortion.b).b, 1 );
}
//---------------------------------------------------------------------------
void main() 
{ 
	vec2 texcoord = -TexCoords + vec2(1.0);
	vec2 texelSize = 1.0 / vec2(textureSize(gSampler, 0));

	// ghost vector to image centre:
	vec2 ghostVec = (vec2(0.5) - texcoord) * gGhostDispersal;
	
	float gDistortion = 1.1;
	vec3 distortion = vec3(-texelSize.x * gDistortion, 0.0, texelSize.x * gDistortion);
	vec2 direction = normalize(ghostVec);

	// sample ghosts:  
	vec4 result = vec4(0.0);
	for (int i = 0; i < gGhosts; ++i) 
	{
		vec2 offset = fract(texcoord + ghostVec * float(i));
		
		float weight = length(vec2(0.5) - offset) / length(vec2(0.5));
		weight = pow(1.0 - weight, 1.0);
  
		result += TextureDistorted(gSampler, offset, direction, distortion) * weight;
	}

	// colourise ghosts:
	result *= texture(gLensColours, vec2(length(vec2(0.5) - texcoord) / length(vec2(0.5)), 0.5));

	// sample halo:
	float gHaloWidth = 0.4;
	float haloIntensity = 0.5;
	vec2 haloVec = normalize(ghostVec) * gHaloWidth;
	float weight = length(vec2(0.5) - fract(texcoord + haloVec)) / length(vec2(0.5));
	weight = pow(1.0 - weight, 5.0);
	result += TextureDistorted(gSampler, texcoord + haloVec, direction, distortion) * weight * haloIntensity;

	outColor = result;
}
