#version 330

const int MAX_POINT_LIGHTS = 3;
const int NUM_CASCADES = 4;

struct BaseLight
{
    vec3  Color;
    float AmbientIntensity;
	float DiffuseIntensity;
};

struct DirectionalLight
{
	BaseLight Base;
	vec3 Direction;
};

struct Attenuation
{
    float Constant;
    float Linear;
    float Exp;
};

struct PointLight
{
    BaseLight Base;
    vec3 Position;
    Attenuation Atten;
	bool ShadowMapEnabled;
	samplerCube ShadowMap;
};
//---------------------------------------------------------------------------
uniform mat4 gMVP;
uniform mat4 gLightMVP[NUM_CASCADES];
uniform mat4 gModelMat;
uniform DirectionalLight gDirLight;
uniform PointLight gPointLights[MAX_POINT_LIGHTS];
uniform vec3 gCameraPos;
uniform int gNumPointLights;
uniform bool gNormalMapEnabled;
uniform float gTextureRepeat;
//---------------------------------------------------------------------------
layout (location = 0) in vec3 inPosition;
layout (location = 1) in vec2 inCoord;
layout (location = 2) in vec3 inNormal;
layout (location = 3) in vec3 inTangent;
layout (location = 4) in vec3 inBitangent;
//---------------------------------------------------------------------------
out vec4 s_LightSpacePos[NUM_CASCADES];
out float s_ClipSpacePosZ;
out vec2 s_TexCoord;
out vec3 s_Normal;
out vec3 s_CameraPos;
out vec3 s_WorldPos;
out vec3 s_Pos;
out vec3 s_DirLightDir;
out vec3 s_PointLightsPos[MAX_POINT_LIGHTS];
//---------------------------------------------------------------------------
void main()
{
	gl_Position = gMVP * vec4(inPosition, 1.0);

	// shadow map
	for (int i = 0 ; i < NUM_CASCADES ; i++) 
	{
		s_LightSpacePos[i] = gLightMVP[i] * vec4(inPosition, 1.0);;
	}
	s_ClipSpacePosZ = gl_Position.z;

	s_TexCoord = inCoord * gTextureRepeat;
	s_WorldPos = (gModelMat * vec4(inPosition, 1.0)).xyz; 

	if( gNormalMapEnabled )
	{
		// Calculate for TBN space
		mat3 normalMatrix = transpose( inverse( mat3(gModelMat) ) );
		vec3 T = normalize(normalMatrix * inTangent);
		vec3 B = normalize(normalMatrix * inBitangent);
		vec3 N = normalize(normalMatrix * inNormal);
    
		mat3 TBN = transpose(mat3(T, B, N));  
 
		s_CameraPos	= TBN * gCameraPos;
		s_Pos		= TBN * s_WorldPos;
		s_Normal	= TBN * (gModelMat * vec4(inNormal, 0.0)).xyz;
	
		// convert light vectors
		s_DirLightDir = TBN * gDirLight.Direction;
		for (int i = 0 ; i < gNumPointLights ; i++) 
		{
			s_PointLightsPos[i] = TBN * gPointLights[i].Position;
		}
	}
	else
	{
		s_CameraPos	= gCameraPos;
		s_Pos		= s_WorldPos;
		s_Normal	= (gModelMat * vec4(inNormal, 0.0)).xyz;
	
		// convert light vectors
		s_DirLightDir = gDirLight.Direction;
		for (int i = 0 ; i < gNumPointLights ; i++) 
		{
			s_PointLightsPos[i] = gPointLights[i].Position;
		}
	}
}