#version 330

layout(points) in;
layout(points) out;
layout(max_vertices = 128) out;

in vec3 s_Position[];
in vec3 s_Velocity[];
in float s_LifeTime[];
in int s_Type[];

out vec3 outPosition;
out vec3 outVelocity;
out float outLifeTime;
out int outType;

uniform vec3 gGenPosition;
uniform vec3 gGenGravityVector;
uniform vec3 gGenVelocityMin;
uniform vec3 gGenVelocityRange;

uniform float gGenLifeMin;
uniform float gGenLifeRange;
uniform float gTimePassed;
uniform float gNoise;

uniform vec3 gRandomSeed;
vec3 m_LocalSeed;

uniform int gGenerateCount;

uniform vec2 gSpawnArea;

float rand()
{
	uint n = floatBitsToUint(m_LocalSeed.y * 214013.0 + m_LocalSeed.x * 2531011.0 + m_LocalSeed.z * 141251.0);
	n = n * (n * n * 15731u + 789221u);
	n = (n >> 9u) | 0x3F800000u;

	float fRes = 2.0 - uintBitsToFloat(n);
	m_LocalSeed = vec3(m_LocalSeed.x + 147158.0 * fRes, m_LocalSeed.y*fRes + 415161.0 * fRes, m_LocalSeed.z + 324154.0*fRes);
	return fRes;
}

void main()
{
	m_LocalSeed = gRandomSeed + s_Velocity[0];

	outPosition = s_Position[0];
	outVelocity = s_Velocity[0];

	if (s_Type[0] != 0)
	{
		vec3 vel = (outVelocity + gGenGravityVector);
		outPosition += vel * gTimePassed;
		outVelocity += vec3(rand()-0.5, (rand()-0.5)*1.0, rand()-0.5) * gNoise * gTimePassed;
	}
	
	outLifeTime = s_LifeTime[0] - gTimePassed;
	outType = s_Type[0];

	if (outType == 0)
	{
		EmitVertex();
		EndPrimitive();

		for (int i = 0; i < gGenerateCount; i++)
		{
			outPosition = gGenPosition + vec3(gSpawnArea.x, 0, gSpawnArea.y) * vec3(rand() - 0.5, 0, rand() - 0.5);
			outVelocity = gGenVelocityMin + gGenVelocityRange * vec3(rand(), rand(), rand());
			outLifeTime = gGenLifeMin + gGenLifeRange * rand();
			outType = 1;
			EmitVertex();
			EndPrimitive();
		}
	}
	else if (outLifeTime > 0.0)
	{
		EmitVertex();
		EndPrimitive();
	}
}
